@@ -18,12 +18,14 @@ import uuid
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import pecan
+import six
 import wsme
 from wsme import types as wtypes
 
 from magnum.api import attr_validator
 from magnum.api.controllers import base
 from magnum.api.controllers import link
+from magnum.api.controllers.v1 import cluster_actions
 from magnum.api.controllers.v1 import collection
 from magnum.api.controllers.v1 import types
 from magnum.api import expose
@@ -110,7 +112,10 @@ class Cluster(base.APIBase):
     docker_volume_size = wtypes.IntegerType(minimum=1)
     """The size in GB of the docker volume"""
 
-    labels = wtypes.DictType(str, str)
+    labels = wtypes.DictType(wtypes.text, types.MultiType(wtypes.text,
+                                                          six.integer_types,
+                                                          bool,
+                                                          float))
     """One or more key/value pairs"""
 
     master_flavor_id = wtypes.StringType(min_length=1, max_length=255)
@@ -128,12 +133,18 @@ class Cluster(base.APIBase):
     stack_id = wsme.wsattr(wtypes.text, readonly=True)
     """Stack id of the heat stack"""
 
-    status = wtypes.Enum(str, *fields.ClusterStatus.ALL)
+    status = wtypes.Enum(wtypes.text, *fields.ClusterStatus.ALL)
     """Status of the cluster from the heat stack"""
 
     status_reason = wtypes.text
     """Status reason of the cluster from the heat stack"""
 
+    health_status = wtypes.Enum(wtypes.text, *fields.ClusterHealthStatus.ALL)
+    """Health status of the cluster from the native COE API"""
+
+    health_status_reason = wtypes.DictType(wtypes.text, wtypes.text)
+    """Health status reason of the cluster from the native COE API"""
+
     discovery_url = wtypes.text
     """Url used for cluster node discovery"""
 
@@ -159,7 +170,7 @@ class Cluster(base.APIBase):
     master_addresses = wsme.wsattr([wtypes.text], readonly=True)
     """IP addresses of cluster master nodes"""
 
-    faults = wsme.wsattr(wtypes.DictType(str, wtypes.text))
+    faults = wsme.wsattr(wtypes.DictType(wtypes.text, wtypes.text))
     """Fault info collected from the heat resources of this cluster"""
 
     def __init__(self, **kwargs):
@@ -180,7 +191,7 @@ class Cluster(base.APIBase):
                                          'labels', 'node_count', 'status',
                                          'master_flavor_id', 'flavor_id',
                                          'create_timeout', 'master_count',
-                                         'stack_id'])
+                                         'stack_id', 'health_status'])
 
         cluster.links = [link.Link.make_link('self', url,
                                              'clusters', cluster.uuid),
@@ -211,6 +222,9 @@ class Cluster(base.APIBase):
                      stack_id='49dc23f5-ffc9-40c3-9d34-7be7f9e34d63',
                      status=fields.ClusterStatus.CREATE_COMPLETE,
                      status_reason="CREATE completed successfully",
+                     health_status=fields.ClusterHealthStatus.HEALTHY,
+                     health_status_reason={"api": "ok",
+                                           "node-0.Ready": 'True'},
                      api_address='172.24.4.3',
                      node_addresses=['172.24.4.4', '172.24.4.5'],
                      created_at=timeutils.utcnow(),
@@ -268,6 +282,8 @@ class ClustersController(base.Controller):
         'detail': ['GET'],
     }
 
+    actions = cluster_actions.ActionsController()
+
     def _generate_name_for_cluster(self, context):
         """Generate a random name like: zeta-22-cluster."""
         name_gen = name_generator.NameGenerator()
@@ -448,7 +464,8 @@ class ClustersController(base.Controller):
             cluster.keypair = cluster_template.keypair_id
 
         # If docker_volume_size is not present, use cluster_template value
-        if cluster.docker_volume_size == wtypes.Unset:
+        if (cluster.docker_volume_size == wtypes.Unset or
+                not cluster.docker_volume_size):
             cluster.docker_volume_size = cluster_template.docker_volume_size
 
         # If labels is not present, use cluster_template value
