@@ -14,6 +14,7 @@
 
 from oslo_utils import timeutils
 import pecan
+import six
 import wsme
 from wsme import types as wtypes
 
@@ -47,7 +48,7 @@ class ClusterTemplate(base.APIBase):
     name = wtypes.StringType(min_length=1, max_length=255)
     """The name of the ClusterTemplate"""
 
-    coe = wtypes.Enum(str, *fields.ClusterType.ALL, mandatory=True)
+    coe = wtypes.Enum(wtypes.text, *fields.ClusterType.ALL, mandatory=True)
     """The Container Orchestration Engine for this clustertemplate"""
 
     image_id = wsme.wsattr(wtypes.StringType(min_length=1, max_length=255),
@@ -112,7 +113,10 @@ class ClusterTemplate(base.APIBase):
     registry_enabled = wsme.wsattr(types.boolean, default=False)
     """Indicates whether the docker registry is enabled"""
 
-    labels = wtypes.DictType(str, str)
+    labels = wtypes.DictType(wtypes.text, types.MultiType(wtypes.text,
+                                                          six.integer_types,
+                                                          bool,
+                                                          float))
     """One or more key/value pairs"""
 
     tls_disabled = wsme.wsattr(types.boolean, default=False)
@@ -121,14 +125,14 @@ class ClusterTemplate(base.APIBase):
     public = wsme.wsattr(types.boolean, default=False)
     """Indicates whether the ClusterTemplate is public or not."""
 
-    server_type = wsme.wsattr(wtypes.Enum(str, *fields.ServerType.ALL),
+    server_type = wsme.wsattr(wtypes.Enum(wtypes.text, *fields.ServerType.ALL),
                               default='vm')
     """Server type for this ClusterTemplate """
 
     insecure_registry = wtypes.StringType(min_length=1, max_length=255)
     """Insecure registry URL when creating a ClusterTemplate """
 
-    docker_storage_driver = wtypes.Enum(str, *fields.DockerStorageDriver.ALL)
+    docker_storage_driver = wtypes.StringType(min_length=1, max_length=255)
     """Docker storage driver"""
 
     master_lb_enabled = wsme.wsattr(types.boolean, default=False)
@@ -139,6 +143,15 @@ class ClusterTemplate(base.APIBase):
     floating_ip_enabled = wsme.wsattr(types.boolean, default=True)
     """Indicates whether created clusters should have a floating ip or not."""
 
+    project_id = wsme.wsattr(wtypes.text, readonly=True)
+    """Project id of the cluster belongs to"""
+
+    user_id = wsme.wsattr(wtypes.text, readonly=True)
+    """User id of the cluster belongs to"""
+
+    hidden = wsme.wsattr(types.boolean, default=False)
+    """Indicates whether the ClusterTemplate is hidden or not."""
+
     def __init__(self, **kwargs):
         self.fields = []
         for field in objects.ClusterTemplate.fields:
@@ -195,7 +208,8 @@ class ClusterTemplate(base.APIBase):
             updated_at=timeutils.utcnow(),
             public=False,
             master_lb_enabled=False,
-            floating_ip_enabled=True)
+            floating_ip_enabled=True,
+            hidden=False)
         return cls._convert_with_links(sample, 'http://localhost:9511')
 
 
@@ -204,7 +218,7 @@ class ClusterTemplatePatchType(types.JsonPatchType):
     _extra_non_removable_attrs = {'/network_driver', '/external_network_id',
                                   '/tls_disabled', '/public', '/server_type',
                                   '/coe', '/registry_enabled',
-                                  '/cluster_distro'}
+                                  '/cluster_distro', '/hidden'}
 
 
 class ClusterTemplateCollection(collection.Collection):
@@ -249,6 +263,23 @@ class ClusterTemplatesController(base.Controller):
                                           sort_key, sort_dir,
                                           resource_url=None):
 
+        context = pecan.request.context
+        if context.is_admin:
+            if resource_url == '/'.join(['clustertemplates', 'detail']):
+                policy.enforce(context, "clustertemplate:detail_all_projects",
+                               action="clustertemplate:detail_all_projects")
+            else:
+                policy.enforce(context, "clustertemplate:get_all_all_projects",
+                               action="clustertemplate:get_all_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So the could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         limit = api_utils.validate_limit(limit)
         sort_dir = api_utils.validate_sort_dir(sort_dir)
 
@@ -317,8 +348,22 @@ class ClusterTemplatesController(base.Controller):
         ClusterTemplate.
         """
         context = pecan.request.context
+
+        if context.is_admin:
+            policy.enforce(context, "clustertemplate:get_one_all_projects",
+                           action="clustertemplate:get_one_all_projects")
+            # TODO(flwang): Instead of asking an extra 'all_project's
+            # parameter, currently the design is allowing admin user to list
+            # all clusters from all projects. But the all_tenants is one of
+            # the condition to do project filter in DB API. And it's also used
+            # by periodic tasks. So the could be removed in the future and
+            # a new parameter 'project_id' would be added so that admin user
+            # can list clusters for a particular project.
+            context.all_tenants = True
+
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
+
         if not cluster_template.public:
             policy.enforce(context, 'clustertemplate:get',
                            cluster_template.as_dict(),
@@ -331,6 +376,7 @@ class ClusterTemplatesController(base.Controller):
     @validation.enforce_network_driver_types_create()
     @validation.enforce_volume_driver_types_create()
     @validation.enforce_volume_storage_size_create()
+    @validation.enforce_driver_supported()
     def post(self, cluster_template):
         """Create a new ClusterTemplate.
 
@@ -348,8 +394,8 @@ class ClusterTemplatesController(base.Controller):
         cluster_template_dict['cluster_distro'] = image_data['os_distro']
         cluster_template_dict['project_id'] = context.project_id
         cluster_template_dict['user_id'] = context.user_id
-        # check permissions for making cluster_template public
-        if cluster_template_dict['public']:
+        # check permissions for making cluster_template public or hidden
+        if cluster_template_dict['public'] or cluster_template_dict['hidden']:
             if not policy.enforce(context, "clustertemplate:publish", None,
                                   do_raise=False):
                 raise exception.ClusterTemplatePublishDenied()
@@ -398,8 +444,9 @@ class ClusterTemplatesController(base.Controller):
         new_cluster_template_dict = new_cluster_template.as_dict()
         attr_validator.validate_os_resources(context,
                                              new_cluster_template_dict)
-        # check permissions when updating ClusterTemplate public flag
-        if cluster_template.public != new_cluster_template.public:
+        # check permissions when updating ClusterTemplate public or hidden flag
+        if (cluster_template.public != new_cluster_template.public or
+                cluster_template.hidden != new_cluster_template.hidden):
             if not policy.enforce(context, "clustertemplate:publish", None,
                                   do_raise=False):
                 raise exception.ClusterTemplatePublishDenied()
@@ -427,6 +474,11 @@ class ClusterTemplatesController(base.Controller):
          ClusterTemplate.
         """
         context = pecan.request.context
+        if context.is_admin:
+            policy.enforce(context, 'clustertemplate:delete_all_projects',
+                           action='clustertemplate:delete_all_projects')
+            context.all_tenants = True
+
         cluster_template = api_utils.get_resource('ClusterTemplate',
                                                   cluster_template_ident)
         policy.enforce(context, 'clustertemplate:delete',
