@@ -238,7 +238,8 @@ class ClusterTemplateReferenced(Invalid):
 
 
 class ClusterTemplatePublishDenied(NotAuthorized):
-    message = _("Not authorized to set public flag for cluster template.")
+    message = _("Not authorized to set public or hidden flag for cluster"
+                " template.")
 
 
 class ClusterNotFound(ResourceNotFound):
@@ -389,5 +390,10 @@ class FederationAlreadyExists(Conflict):
 
 
 class MemberAlreadyExists(Conflict):
-    message = _("A cluster with UUID %(uuid)s is already a member of the"
+    message = _("A cluster with UUID %(uuid)s is already a member of the "
                 "federation %(federation_name)s.")
+
+
+class PreDeletionFailed(Conflict):
+    message = _("Failed to pre-delete resources for cluster %(cluster_uuid)s, "
+                "error: %(msg)s.")
