@@ -16,10 +16,9 @@
 SQLAlchemy models for container service
 """
 
-import json
-
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy.types import String
+from oslo_serialization import jsonutils
 import six.moves.urllib.parse as urlparse
 from sqlalchemy import Boolean
 from sqlalchemy import Column
@@ -61,12 +60,12 @@ class JsonEncodedType(TypeDecorator):
                             "given" % {'class': self.__class__.__name__,
                                        'type': self.type.__name__,
                                        'value': type(value).__name__})
-        serialized_value = json.dumps(value)
+        serialized_value = jsonutils.dumps(value)
         return serialized_value
 
     def process_result_value(self, value, dialect):
         if value is not None:
-            value = json.loads(value)
+            value = jsonutils.loads(value)
         return value
 
 
@@ -128,6 +127,8 @@ class Cluster(Base):
     master_count = Column(Integer())
     status = Column(String(20))
     status_reason = Column(Text)
+    health_status = Column(String(20))
+    health_status_reason = Column(JSONEncodedDict)
     create_timeout = Column(Integer())
     discovery_url = Column(String(255, mysql_ndb_type=TINYTEXT))
     master_addresses = Column(JSONEncodedList)
@@ -188,6 +189,7 @@ class ClusterTemplate(Base):
     insecure_registry = Column(String(255, mysql_ndb_type=TINYTEXT))
     master_lb_enabled = Column(Boolean, default=False)
     floating_ip_enabled = Column(Boolean, default=True)
+    hidden = Column(Boolean, default=False)
 
 
 class X509KeyPair(Base):
