@@ -1,5 +1,8 @@
 #!/bin/sh
 
+step="core-dns-service"
+printf "Starting to run ${step}\n"
+
 . /etc/sysconfig/heat-params
 
 _dns_prefix=${CONTAINER_INFRA_PREFIX:-docker.io/coredns/}
@@ -65,6 +68,7 @@ data:
         kubernetes ${DNS_CLUSTER_DOMAIN} ${PORTAL_NETWORK_CIDR} ${PODS_NETWORK_CIDR} {
             pods verified
         }
+        prometheus :9153
         proxy . /etc/resolv.conf
         cache 30
     }
@@ -95,8 +99,8 @@ spec:
           operator: "Exists"
       containers:
       - name: coredns
-        image: ${_dns_prefix}coredns:1.0.1
-        imagePullPolicy: Always
+        image: ${_dns_prefix}coredns:1.3.0
+        imagePullPolicy: IfNotPresent
         args: [ "-conf", "/etc/coredns/Corefile" ]
         volumeMounts:
         - name: config-volume
@@ -245,15 +249,6 @@ spec:
 EOF
 }
 
-# NOTE(flwang): Let's keep the same addons yaml file on all masters,
-# but if it's not the primary/bootstrapping master, don't try to
-# create those resources to avoid race condition issue until the
-# kubectl issue https://github.com/kubernetes/kubernetes/issues/44165
-# fixed.
-if [ "$MASTER_INDEX" != "0" ]; then
-    exit 0
-fi
-
 echo "Waiting for Kubernetes API..."
 until  [ "ok" = "$(curl --silent http://127.0.0.1:8080/healthz)" ]
 do
@@ -261,3 +256,5 @@ do
 done
 
 kubectl apply --validate=false -f $CORE_DNS
+
+printf "Finished running ${step}\n"
