@@ -68,7 +68,7 @@ EOF
     content_type='Content-Type: application/json'
     url="$AUTH_URL/auth/tokens"
     USER_TOKEN=`curl $VERIFY_CA -s -i -X POST -H "$content_type" -d "$auth_json" $url \
-        | grep X-Subject-Token | awk '{print $2}' | tr -d '[[:space:]]'`
+        | grep -i X-Subject-Token | awk '{print $2}' | tr -d '[[:space:]]'`
 
     # Get CA certificate for this cluster
     curl $VERIFY_CA -X GET \
@@ -147,5 +147,3 @@ chmod 550 "${cert_dir}"
 chown -R kube:kube_etcd "${cert_dir}"
 chmod 440 ${cert_dir}/kubelet.key
 chmod 440 ${cert_dir}/proxy.key
-mkdir -p /etc/flanneld/certs
-cp ${cert_dir}/* /etc/flanneld/certs
