@@ -15,8 +15,11 @@ import abc
 import ast
 
 from oslo_log import log as logging
+from oslo_utils import strutils
+import re
 import requests
 import six
+from six.moves.urllib import parse as urlparse
 
 from magnum.common import clients
 from magnum.common import exception
@@ -108,12 +111,12 @@ class OutputMapping(object):
 
 @six.add_metaclass(abc.ABCMeta)
 class TemplateDefinition(object):
-    '''A mapping between Magnum objects and Heat templates.
+    """A mapping between Magnum objects and Heat templates.
 
     A TemplateDefinition is essentially a mapping between Magnum objects
     and Heat templates. Each TemplateDefinition has a mapping of Heat
     parameters.
-    '''
+    """
 
     def __init__(self):
         self.param_mappings = list()
@@ -210,8 +213,7 @@ class BaseTemplateDefinition(TemplateDefinition):
         self._osc = None
 
         self.add_parameter('ssh_key_name',
-                           cluster_attr='keypair',
-                           required=True)
+                           cluster_attr='keypair')
         self.add_parameter('server_image',
                            cluster_template_attr='image_id')
         self.add_parameter('dns_nameserver',
@@ -262,7 +264,7 @@ class BaseTemplateDefinition(TemplateDefinition):
         }
         if CONF.trust.trustee_keystone_region_name:
             kwargs['region_name'] = CONF.trust.trustee_keystone_region_name
-        extra_params['auth_url'] = osc.url_for(**kwargs)
+        extra_params['auth_url'] = osc.url_for(**kwargs).rstrip('/')
 
         return super(BaseTemplateDefinition,
                      self).get_params(context, cluster_template, cluster,
@@ -279,7 +281,7 @@ class BaseTemplateDefinition(TemplateDefinition):
         try:
             result = requests.get(url).text
         except req_exceptions.RequestException as err:
-            LOG.error(six.text_type(err))
+            LOG.error(err)
             raise exception.GetClusterSizeFailed(
                 discovery_url=discovery_url)
 
@@ -304,7 +306,26 @@ class BaseTemplateDefinition(TemplateDefinition):
                 size=int(value),
                 discovery_url=discovery_url)
 
-    def get_discovery_url(self, cluster):
+    def get_proxies(self, url, cluster_template):
+        proxies = dict()
+        if cluster_template is None:
+            return proxies
+        hostname = urlparse.urlparse(url).netloc.partition(":")[0]
+        if hasattr(cluster_template, 'no_proxy') and \
+            cluster_template.no_proxy and \
+            (cluster_template.no_proxy == '*' or
+             re.search('\\b%s\\b' % re.escape(hostname),
+                       cluster_template.no_proxy, re.I)):
+            LOG.debug('Bypass proxy, because discovery hostname is listed in'
+                      ' cluster template no_proxy variable')
+        else:
+            if hasattr(cluster_template, 'http_proxy'):
+                proxies['http'] = cluster_template.http_proxy
+            if hasattr(cluster_template, 'https_proxy'):
+                proxies['https'] = cluster_template.https_proxy
+        return proxies
+
+    def get_discovery_url(self, cluster, cluster_template=None):
         if hasattr(cluster, 'discovery_url') and cluster.discovery_url:
             if getattr(cluster, 'master_count', None) is not None:
                 self.validate_discovery_url(cluster.discovery_url,
@@ -317,13 +338,16 @@ class BaseTemplateDefinition(TemplateDefinition):
                 CONF.cluster.etcd_discovery_service_endpoint_format %
                 {'size': cluster.master_count})
             try:
-                discovery_request = requests.get(discovery_endpoint)
+                proxies = self.get_proxies(discovery_endpoint,
+                                           cluster_template)
+                discovery_request = requests.get(discovery_endpoint,
+                                                 proxies=proxies)
                 if discovery_request.status_code != requests.codes.ok:
                     raise exception.GetDiscoveryUrlFailed(
                         discovery_endpoint=discovery_endpoint)
                 discovery_url = discovery_request.text
             except req_exceptions.RequestException as err:
-                LOG.error(six.text_type(err))
+                LOG.error(err)
                 raise exception.GetDiscoveryUrlFailed(
                     discovery_endpoint=discovery_endpoint)
             if not discovery_url:
@@ -334,6 +358,9 @@ class BaseTemplateDefinition(TemplateDefinition):
                 cluster.discovery_url = discovery_url
         return discovery_url
 
+    def get_scale_params(self, context, cluster, scale_manager=None):
+        return dict()
+
 
 def add_lb_env_file(env_files, cluster_template):
     if cluster_template.master_lb_enabled:
@@ -359,15 +386,22 @@ def add_etcd_volume_env_file(env_files, cluster_template):
         env_files.append(COMMON_ENV_PATH + 'with_etcd_volume.yaml')
 
 
-def add_fip_env_file(env_files, cluster_template):
+def add_fip_env_file(env_files, cluster_template, cluster):
+    lb_fip_enabled = cluster.labels.get(
+        "master_lb_floating_ip_enabled",
+        cluster_template.floating_ip_enabled
+    )
+    master_lb_fip_enabled = strutils.bool_from_string(lb_fip_enabled)
+
     if cluster_template.floating_ip_enabled:
         env_files.append(COMMON_ENV_PATH + 'enable_floating_ip.yaml')
-        if cluster_template.master_lb_enabled:
-            env_files.append(COMMON_ENV_PATH + 'enable_lb_floating_ip.yaml')
     else:
         env_files.append(COMMON_ENV_PATH + 'disable_floating_ip.yaml')
-        if cluster_template.master_lb_enabled:
-            env_files.append(COMMON_ENV_PATH + 'disable_lb_floating_ip.yaml')
+
+    if cluster_template.master_lb_enabled and master_lb_fip_enabled:
+        env_files.append(COMMON_ENV_PATH + 'enable_lb_floating_ip.yaml')
+    else:
+        env_files.append(COMMON_ENV_PATH + 'disable_lb_floating_ip.yaml')
 
 
 def add_priv_net_env_file(env_files, cluster_template):
