@@ -13,8 +13,11 @@
 # under the License.
 import os
 
+import magnum.conf
 from magnum.drivers.heat import template_def
 
+CONF = magnum.conf.CONF
+
 
 class UbuntuMesosTemplateDefinition(template_def.BaseTemplateDefinition):
     """Mesos template for Ubuntu VM."""
@@ -61,6 +64,8 @@ class UbuntuMesosTemplateDefinition(template_def.BaseTemplateDefinition):
         extra_params['tenant_name'] = context.project_id
         extra_params['domain_name'] = context.domain_name
         extra_params['region_name'] = osc.cinder_region_name()
+        extra_params['nodes_affinity_policy'] = \
+            CONF.cluster.nodes_affinity_policy
 
         label_list = ['rexray_preempt', 'mesos_slave_isolation',
                       'mesos_slave_image_providers',
@@ -70,17 +75,22 @@ class UbuntuMesosTemplateDefinition(template_def.BaseTemplateDefinition):
         for label in label_list:
             extra_params[label] = cluster.labels.get(label)
 
-        scale_mgr = kwargs.pop('scale_manager', None)
-        if scale_mgr:
-            hosts = self.get_output('mesos_slaves_private')
-            extra_params['slaves_to_remove'] = (
-                scale_mgr.get_removal_nodes(hosts))
-
         return super(UbuntuMesosTemplateDefinition,
                      self).get_params(context, cluster_template, cluster,
                                       extra_params=extra_params,
                                       **kwargs)
 
+    def get_scale_params(self, context, cluster, scale_manager=None,
+                         nodes_to_remove=None):
+        scale_params = dict()
+        if nodes_to_remove:
+            scale_params['slaves_to_remove'] = nodes_to_remove
+        if scale_manager:
+            hosts = self.get_output('mesos_slaves_private')
+            scale_params['slaves_to_remove'] = (
+                scale_manager.get_removal_nodes(hosts))
+        return scale_params
+
     def get_env_files(self, cluster_template, cluster):
         env_files = []
 
