@@ -463,8 +463,10 @@ class TestHandler(db_base.DbTestCase):
     @patch('magnum.conductor.handlers.cluster_conductor.cert_manager')
     @patch('magnum.common.clients.OpenStackClients')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
-    def test_cluster_delete(self, mock_driver, mock_openstack_client_class,
-                            cert_manager):
+    @patch('magnum.common.keystone.is_octavia_enabled')
+    def test_cluster_delete(self, mock_octavia, mock_driver,
+                            mock_openstack_client_class, cert_manager):
+        mock_octavia.return_value = False
         mock_driver.return_value = k8s_atomic_dr.Driver()
         osc = mock.MagicMock()
         mock_openstack_client_class.return_value = osc
@@ -490,9 +492,11 @@ class TestHandler(db_base.DbTestCase):
     @patch('magnum.conductor.handlers.cluster_conductor.cert_manager')
     @patch('magnum.common.clients.OpenStackClients')
     @patch('magnum.drivers.common.driver.Driver.get_driver')
-    def test_cluster_delete_conflict(self, mock_driver,
+    @patch('magnum.common.keystone.is_octavia_enabled')
+    def test_cluster_delete_conflict(self, mock_octavia, mock_driver,
                                      mock_openstack_client_class,
                                      cert_manager):
+        mock_octavia.return_value = False
         mock_driver.return_value = k8s_atomic_dr.Driver()
         osc = mock.MagicMock()
         mock_openstack_client_class.return_value = osc
@@ -514,3 +518,74 @@ class TestHandler(db_base.DbTestCase):
             taxonomy.OUTCOME_FAILURE, notifications[1].payload['outcome'])
         self.assertEqual(
             0, cert_manager.delete_certificates_from_cluster.call_count)
+
+    @patch('magnum.drivers.common.driver.Driver.get_driver')
+    @patch('magnum.common.clients.OpenStackClients')
+    @patch('magnum.common.keystone.is_octavia_enabled')
+    @patch('magnum.common.octavia.delete_loadbalancers')
+    def test_cluster_delete_with_lb(self, mock_delete_lb, mock_octavia,
+                                    mock_clients, mock_driver):
+        mock_octavia.return_value = True
+        mock_driver.return_value = k8s_atomic_dr.Driver()
+
+        self.handler.cluster_delete(self.context, self.cluster.uuid)
+
+        notifications = fake_notifier.NOTIFICATIONS
+        self.assertEqual(1, len(notifications))
+        self.assertEqual(1, mock_delete_lb.call_count)
+
+    @patch('magnum.conductor.scale_manager.get_scale_manager')
+    @patch('magnum.drivers.common.driver.Driver.get_driver')
+    @patch('magnum.common.clients.OpenStackClients')
+    def test_cluster_resize_success(
+            self, mock_openstack_client_class,
+            mock_driver,
+            mock_scale_manager):
+
+        mock_heat_stack = mock.MagicMock()
+        mock_heat_stack.stack_status = cluster_status.CREATE_COMPLETE
+        mock_heat_client = mock.MagicMock()
+        mock_heat_client.stacks.get.return_value = mock_heat_stack
+        mock_openstack_client = mock_openstack_client_class.return_value
+        mock_openstack_client.heat.return_value = mock_heat_client
+        mock_dr = mock.MagicMock()
+        mock_driver.return_value = mock_dr
+
+        self.cluster.status = cluster_status.CREATE_COMPLETE
+        self.handler.cluster_resize(self.context, self.cluster, 3, ["ID1"])
+
+        notifications = fake_notifier.NOTIFICATIONS
+        self.assertEqual(1, len(notifications))
+        self.assertEqual(
+            'magnum.cluster.update', notifications[0].event_type)
+        self.assertEqual(
+            taxonomy.OUTCOME_PENDING, notifications[0].payload['outcome'])
+
+        mock_dr.resize_cluster.assert_called_once_with(
+            self.context, self.cluster, mock_scale_manager.return_value, 3,
+            ["ID1"], None)
+
+    @patch('magnum.common.clients.OpenStackClients')
+    def test_cluster_resize_failure(
+            self, mock_openstack_client_class):
+
+        mock_heat_stack = mock.MagicMock()
+        mock_heat_stack.stack_status = cluster_status.CREATE_FAILED
+        mock_heat_client = mock.MagicMock()
+        mock_heat_client.stacks.get.return_value = mock_heat_stack
+        mock_openstack_client = mock_openstack_client_class.return_value
+        mock_openstack_client.heat.return_value = mock_heat_client
+
+        self.cluster.status = cluster_status.CREATE_FAILED
+        self.assertRaises(exception.NotSupported, self.handler.cluster_resize,
+                          self.context, self.cluster, 2, [])
+
+        notifications = fake_notifier.NOTIFICATIONS
+        self.assertEqual(1, len(notifications))
+        self.assertEqual(
+            'magnum.cluster.update', notifications[0].event_type)
+        self.assertEqual(
+            taxonomy.OUTCOME_FAILURE, notifications[0].payload['outcome'])
+
+        cluster = objects.Cluster.get(self.context, self.cluster.uuid)
+        self.assertEqual(1, cluster.node_count)
