@@ -33,7 +33,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'master_flavor_id': 'master_flavor_id',
             'keypair_id': 'keypair_id',
             'dns_nameserver': 'dns_nameserver',
-            'external_network_id': 'external_network_id',
+            'external_network_id': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'network_driver': 'network_driver',
@@ -59,7 +59,8 @@ class TestClusterConductorWithK8s(base.TestCase):
                        'influx_grafana_dashboard_enabled': 'True',
                        'docker_volume_type': 'lvmdriver-1',
                        'etcd_volume_size': 0,
-                       'availability_zone': 'az_1'},
+                       'availability_zone': 'az_1',
+                       'service_cluster_ip_range': '10.254.0.0/16'},
             'tls_disabled': False,
             'server_type': 'vm',
             'registry_enabled': False,
@@ -107,9 +108,11 @@ class TestClusterConductorWithK8s(base.TestCase):
                        'kubecontroller_options': '--kubecontroller',
                        'kubescheduler_options': '--kubescheduler',
                        'kubeproxy_options': '--kubeproxy',
-                       'influx_grafana_dashboard_enabled': 'True'},
+                       'influx_grafana_dashboard_enabled': 'True',
+                       'service_cluster_ip_range': '10.254.0.0/16'},
             'master_flavor_id': 'master_flavor_id',
             'flavor_id': 'flavor_id',
+            'project_id': 'project_id',
         }
         self.context.user_name = 'fake_user'
         self.context.project_id = 'fake_tenant'
@@ -217,6 +220,7 @@ class TestClusterConductorWithK8s(base.TestCase):
                        'kubecontroller_options': '--kubecontroller',
                        'kubescheduler_options': '--kubescheduler',
                        'kubeproxy_options': '--kubeproxy',
+                       'service_cluster_ip_range': '10.254.0.0/16',
                        },
             'http_proxy': 'http_proxy',
             'https_proxy': 'https_proxy',
@@ -228,7 +232,7 @@ class TestClusterConductorWithK8s(base.TestCase):
         }
         expected = {
             'ssh_key_name': 'keypair_id',
-            'external_network': 'external_network_id',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'network_driver': 'network_driver',
@@ -278,6 +282,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
@@ -286,6 +291,8 @@ class TestClusterConductorWithK8s(base.TestCase):
             'octavia_enabled': False,
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
+            'portal_network_cidr': '10.254.0.0/16',
+            'project_id': 'project_id'
         }
         if missing_attr is not None:
             expected.pop(mapping[missing_attr], None)
@@ -297,6 +304,7 @@ class TestClusterConductorWithK8s(base.TestCase):
              '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
              '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml',
              ],
             env_files)
 
@@ -352,7 +360,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'docker_volume_size': 20,
             'docker_volume_type': 'lvmdriver-1',
             'etcd_volume_size': None,
-            'external_network': 'external_network_id',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'flannel_backend': 'vxlan',
@@ -397,6 +405,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
@@ -405,6 +414,8 @@ class TestClusterConductorWithK8s(base.TestCase):
             'octavia_enabled': False,
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
+            'portal_network_cidr': '10.254.0.0/16',
+            'project_id': 'project_id'
         }
 
         self.assertEqual(expected, definition)
@@ -414,6 +425,7 @@ class TestClusterConductorWithK8s(base.TestCase):
              '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
              '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml'
              ],
             env_files)
 
@@ -468,7 +480,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'docker_volume_size': 20,
             'master_flavor': 'master_flavor_id',
             'minion_flavor': 'flavor_id',
-            'external_network': 'external_network_id',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'flannel_backend': 'vxlan',
             'flannel_network_cidr': '10.101.0.0/16',
             'flannel_network_subnetlen': '26',
@@ -503,6 +515,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
@@ -511,6 +524,8 @@ class TestClusterConductorWithK8s(base.TestCase):
             'octavia_enabled': False,
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
+            'portal_network_cidr': '10.254.0.0/16',
+            'project_id': 'project_id'
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
@@ -519,6 +534,7 @@ class TestClusterConductorWithK8s(base.TestCase):
              '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
              '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml'
              ],
             env_files)
 
@@ -551,10 +567,15 @@ class TestClusterConductorWithK8s(base.TestCase):
 
         expected = {
             'ssh_key_name': 'keypair_id',
-            'external_network': 'external_network_id',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
+            'availability_zone': 'az_1',
+            'nodes_affinity_policy': 'soft-anti-affinity',
             'dns_nameserver': 'dns_nameserver',
+            'docker_storage_driver': 'devicemapper',
+            'docker_volume_size': 20,
+            'docker_volume_type': 'lvmdriver-1',
             'server_image': 'image_id',
             'minion_flavor': 'flavor_id',
             'master_flavor': 'master_flavor_id',
@@ -574,6 +595,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'system_pods_timeout': '1',
             'admission_control_list': 'fake_list',
             'prometheus_monitoring': 'False',
+            'region_name': 'RegionOne',
             'grafana_admin_passwd': 'fake_pwd',
             'kube_dashboard_enabled': 'True',
             'influx_grafana_dashboard_enabled': 'True',
@@ -583,6 +605,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'trustee_username': 'fake_trustee',
             'trustee_password': 'fake_trustee_password',
             'trustee_user_id': '7b489f04-b458-4541-8179-6a48a553e656',
+            'username': 'fake_user',
             'trust_id': '',
             'auth_url': 'http://192.168.10.10:5000/v3',
             'cluster_uuid': self.cluster_dict['uuid'],
@@ -591,21 +614,28 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_version': 'fake-version',
             'verify_ca': True,
             'openstack_ca': '',
+            'openstack_ca_coreos': '',
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
             'kubescheduler_options': '--kubescheduler',
             'kubeproxy_options': '--kubeproxy',
             'octavia_enabled': False,
+            'portal_network_cidr': '10.254.0.0/16',
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
             ['../../common/templates/environments/no_private_network.yaml',
+             '../../common/templates/environments/no_etcd_volume.yaml',
+             '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
-             '../../common/templates/environments/disable_floating_ip.yaml'],
+             '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml'
+             ],
             env_files)
 
     @patch('requests.get')
@@ -635,10 +665,14 @@ class TestClusterConductorWithK8s(base.TestCase):
 
         expected = {
             'ssh_key_name': 'keypair_id',
-            'external_network': 'external_network_id',
+            'availability_zone': 'az_1',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'dns_nameserver': 'dns_nameserver',
+            'docker_storage_driver': u'devicemapper',
+            'docker_volume_size': 20,
+            'docker_volume_type': u'lvmdriver-1',
             'server_image': 'image_id',
             'minion_flavor': 'flavor_id',
             'master_flavor': 'master_flavor_id',
@@ -651,6 +685,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'http_proxy': 'http_proxy',
             'https_proxy': 'https_proxy',
             'no_proxy': 'no_proxy',
+            'nodes_affinity_policy': 'soft-anti-affinity',
             'flannel_network_cidr': '10.101.0.0/16',
             'flannel_network_subnetlen': '26',
             'flannel_backend': 'vxlan',
@@ -658,6 +693,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'system_pods_timeout': '1',
             'admission_control_list': 'fake_list',
             'prometheus_monitoring': 'False',
+            'region_name': self.mock_osc.cinder_region_name.return_value,
             'grafana_admin_passwd': 'fake_pwd',
             'kube_dashboard_enabled': 'True',
             'influx_grafana_dashboard_enabled': 'True',
@@ -665,6 +701,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'registry_enabled': False,
             'trustee_domain_id': self.mock_keystone.trustee_domain_id,
             'trustee_username': 'fake_trustee',
+            'username': 'fake_user',
             'trustee_password': 'fake_trustee_password',
             'trustee_user_id': '7b489f04-b458-4541-8179-6a48a553e656',
             'trust_id': '',
@@ -675,21 +712,28 @@ class TestClusterConductorWithK8s(base.TestCase):
             'kube_version': 'fake-version',
             'verify_ca': True,
             'openstack_ca': '',
+            'openstack_ca_coreos': '',
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
             'kubescheduler_options': '--kubescheduler',
             'kubeproxy_options': '--kubeproxy',
             'octavia_enabled': False,
+            'portal_network_cidr': '10.254.0.0/16',
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
             ['../../common/templates/environments/no_private_network.yaml',
+             '../../common/templates/environments/no_etcd_volume.yaml',
+             '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
-             '../../common/templates/environments/disable_floating_ip.yaml'],
+             '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml'
+             ],
             env_files)
 
     @patch('requests.get')
@@ -864,7 +908,7 @@ class TestClusterConductorWithK8s(base.TestCase):
 
         expected = {
             'ssh_key_name': 'keypair_id',
-            'external_network': 'external_network_id',
+            'external_network': 'e2a6c8b0-a3c2-42a3-b3f4-01400a30896e',
             'fixed_network': 'fixed_network',
             'fixed_subnet': 'fixed_subnet',
             'dns_nameserver': 'dns_nameserver',
@@ -914,6 +958,7 @@ class TestClusterConductorWithK8s(base.TestCase):
             'cert_manager_api': 'False',
             'ingress_controller': 'i-controller',
             'ingress_controller_role': 'i-controller-role',
+            'octavia_ingress_controller_tag': None,
             'kubelet_options': '--kubelet',
             'kubeapi_options': '--kubeapi',
             'kubecontroller_options': '--kubecontroller',
@@ -922,6 +967,8 @@ class TestClusterConductorWithK8s(base.TestCase):
             'octavia_enabled': False,
             'kube_service_account_key': 'public_key',
             'kube_service_account_private_key': 'private_key',
+            'portal_network_cidr': '10.254.0.0/16',
+            'project_id': 'project_id'
         }
         self.assertEqual(expected, definition)
         self.assertEqual(
@@ -930,9 +977,11 @@ class TestClusterConductorWithK8s(base.TestCase):
              '../../common/templates/environments/with_volume.yaml',
              '../../common/templates/environments/no_master_lb.yaml',
              '../../common/templates/environments/disable_floating_ip.yaml',
+             '../../common/templates/environments/disable_lb_floating_ip.yaml',
              ],
             env_files)
-        reqget.assert_called_once_with('http://etcd/test?size=1')
+        reqget.assert_called_once_with('http://etcd/test?size=1', proxies={
+            'http': 'http_proxy', 'https': 'https_proxy'})
 
     @patch('magnum.common.short_id.generate_id')
     @patch('heatclient.common.template_utils.get_template_contents')
@@ -1066,13 +1115,6 @@ class TestClusterConductorWithK8s(base.TestCase):
                           mock_get_template_contents):
 
         mock_stack_id = 'xx-xx-xx-xx'
-        expected_template_contents = 'template_contents'
-
-        mock_tpl_files = {}
-        mock_get_template_contents.return_value = [
-            mock_tpl_files, expected_template_contents]
-        mock_extract_template_definition.return_value = ('template/path',
-                                                         {}, [])
         mock_heat_client = mock.MagicMock()
         mock_osc.return_value.heat.return_value = mock_heat_client
         mock_cluster = mock.MagicMock()
@@ -1081,10 +1123,8 @@ class TestClusterConductorWithK8s(base.TestCase):
         k8s_dr.Driver().update_cluster({}, mock_cluster)
 
         expected_args = {
-            'parameters': {},
-            'template': expected_template_contents,
-            'files': {},
-            'environment_files': [],
+            'parameters': {'number_of_minions': mock_cluster.node_count},
+            'existing': True,
             'disable_rollback': True
         }
         mock_heat_client.stacks.update.assert_called_once_with(mock_stack_id,
